@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2004 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2004, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -23,7 +23,8 @@ import org.eclipse.draw2d.geometry.PointList;
 
 /**
  * A Path representation for the ShortestPathRouting. A Path has a start and end point 
- * and may have bendpoints. The output of a path is accessed via the method <code>getPoints()</code>.
+ * and may have bendpoints. The output of a path is accessed via the method
+ * <code>getPoints()</code>.
  * 
  * This class is for internal use only.
  * @author Whitney Sorenson
@@ -49,7 +50,10 @@ void push(Object obj) {
 }
 
 }
+
+private static final Point CURRENT = new Point();
 private static final double EPSILON = 1.04;
+private static final Point NEXT = new Point();
 
 private static final double OVAL_CONSTANT = 1.13;
 
@@ -58,21 +62,29 @@ private static final double OVAL_CONSTANT = 1.13;
  */
 public Object data;
 
-private static final Point CURRENT = new Point();
-private static final Point NEXT = new Point();
+List grownSegments;
+List segments;
+List excludedObstacles;
 
-private double distance;
-List grownSegments, segments, excludedObstacles;
 /**
  * this field is for internal use only.  It is true whenever a property has been changed
- * which requires the solver to reroute this path.
+ * which requires the solver to resolve this path.
  */
 public boolean isDirty = true;
 boolean isInverted = false;
 boolean isMarked = false;
 
-PointList points, bendpoints;
-private double prevCost;
+PointList points;
+/**
+ * The bendpoint constraints.  The path must go through these bendpoints.
+ */
+PointList bendpoints;
+
+/**
+ * The previous cost ratio of the path.  The cost ratio is the actual path length divided
+ * by the length from the start to the end. 
+ */
+private double prevCostRatio;
 private SegmentStack stack;
 Vertex start, end;
 private Path subPath;
@@ -80,7 +92,6 @@ double threshold;
 Set visibleObstacles;
 Set visibleVertices;
 
-
 /**
  * Constructs a new path.
  * @since 3.0
@@ -133,39 +144,69 @@ Path(Vertex start, Vertex end) {
  * @param target the target obstacle
  */
 private void addAllSegmentsBetween(Obstacle source, Obstacle target) {
-	addConnectingSegment(new Segment(source.bottomLeft, target.bottomLeft), source, target, false);
-	addConnectingSegment(new Segment(source.bottomLeft, target.bottomRight), source, target, false);
-	addConnectingSegment(new Segment(source.bottomRight, target.bottomLeft), source, target, true);
-	addConnectingSegment(new Segment(source.bottomRight, target.bottomRight), source, target, true);
-	addConnectingSegment(new Segment(source.topLeft, target.topRight),  source, target, true);
-	addConnectingSegment(new Segment(source.topLeft, target.topLeft), source, target, true);
-	addConnectingSegment(new Segment(source.topRight, target.topRight), source, target, false);
-	addConnectingSegment(new Segment(source.topRight, target.topLeft), source, target, false);
+	addConnectingSegment(new Segment(source.bottomLeft, target.bottomLeft),
+			source, target, false, false);
+	addConnectingSegment(new Segment(source.bottomRight, target.bottomRight),
+			source, target, true, true);
+	addConnectingSegment(new Segment(source.topLeft, target.topLeft),
+			source, target, true, true);
+	addConnectingSegment(new Segment(source.topRight, target.topRight),
+			source, target, false, false);
+
+	if (source.bottom() == target.bottom()) {
+		addConnectingSegment(new Segment(source.bottomLeft, target.bottomRight),
+				source, target, false, true);
+		addConnectingSegment(new Segment(source.bottomRight, target.bottomLeft),
+				source, target, true, false);
+	}
+	if (source.y == target.y) {
+		addConnectingSegment(new Segment(source.topLeft, target.topRight),
+				source, target, true, false);
+		addConnectingSegment(new Segment(source.topRight, target.topLeft),
+				source, target, false, true);
+	}
+	if (source.x == target.x) {
+		addConnectingSegment(new Segment(source.bottomLeft, target.topLeft),
+				source, target, false, true);
+		addConnectingSegment(new Segment(source.topLeft, target.bottomLeft),
+				source, target, true, false);
+	}
+	if (source.right() == target.right()) {
+		addConnectingSegment(new Segment(source.bottomRight, target.topRight),
+				source, target, true, false);
+		addConnectingSegment(new Segment(source.topRight, target.bottomRight),
+				source, target, false, true);
+	}
 }
 
 /**
  * Attempts to add a segment between the given obstacles to the visibility graph. This 
  * method is specifically written for the case where the two obstacles intersect and contains
- * a boolean as to whether to check the diagnol that includes the top right point of the other obstacle.
+ * a boolean as to whether to check the diagonal that includes the top right point of the
+ * other obstacle.
  * 
  * @param segment the segment to check
  * @param o1 the first obstacle
  * @param o2 the second obstacle
- * @param checkTopRight whether or not to check the diagnol containing top right point
+ * @param checkTopRight1 whether or not to check the diagonal containing top right point
  */
-private void addConnectingSegment(Segment segment, Obstacle o1, Obstacle o2, boolean checkTopRight) {
+private void addConnectingSegment(Segment segment, Obstacle o1, Obstacle o2,
+		boolean checkTopRight1, boolean checkTopRight2) {
 	if (threshold != 0
 			&& (segment.end.getDistance(end) + segment.end.getDistance(start) > threshold
 				|| segment.start.getDistance(end) + segment.start.getDistance(start) > threshold))
 		return;
-	if (o2.contains(segment.start) || o1.contains(segment.end)) 
+	
+	if (o2.containsProper(segment.start) || o1.containsProper(segment.end)) 
 		return;
 	
-	if (checkTopRight && (segment.intersects(o1.x, o1.bottom() - 1, o1.right() - 1, o1.y) 
-			|| segment.intersects(o2.x, o2.bottom() - 1, o2.right() - 1, o2.y)))
+	if (checkTopRight1 && segment.intersects(o1.x, o1.bottom() - 1, o1.right() - 1, o1.y))
 		return;
-	if (!checkTopRight && (segment.intersects(o1.x, o1.y, o1.right() - 1, o1.bottom() - 1)
-			|| segment.intersects(o2.x, o2.y, o2.right() - 1, o2.bottom() - 1)))
+	if (checkTopRight2 && segment.intersects(o2.x, o2.bottom() - 1, o2.right() - 1, o2.y))
+		return;
+	if (!checkTopRight1 && segment.intersects(o1.x, o1.y, o1.right() - 1, o1.bottom() - 1))
+		return;
+	if (!checkTopRight2 && segment.intersects(o2.x, o2.y, o2.right() - 1, o2.bottom() - 1))
 		return;
 	
 	stack.push(o1);
@@ -237,8 +278,8 @@ private void addSegment(Segment segment, Obstacle exclude1, Obstacle exclude2, L
 
 		if (segment.intersects(obs.x, obs.y, obs.right() - 1, obs.bottom() - 1)
 				|| segment.intersects(obs.x, obs.bottom() - 1, obs.right() - 1, obs.y)
-				|| obs.contains(segment.start) 
-				|| obs.contains(segment.end)) {
+				|| obs.containsProper(segment.start) 
+				|| obs.containsProper(segment.end)) {
 			if (!visibleObstacles.contains(obs))
 				addObstacle(obs);
 			return;
@@ -257,13 +298,13 @@ private void addSegmentsFor(Obstacle source, Obstacle target) {
 	if (source.intersects(target))
 		addAllSegmentsBetween(source, target);
 	else if (target.bottom() - 1 < source.y)
-		targetAboveSource(source, target);
+		addSegmentsTargetAboveSource(source, target);
 	else if (source.bottom() - 1 < target.y)
-		targetAboveSource(target, source);
+		addSegmentsTargetAboveSource(target, source);
 	else if (target.right() - 1 < source.x)
-		targetBesideSource(source, target);
+		addSegmentsTargetBesideSource(source, target);
 	else
-		targetBesideSource(target, source);
+		addSegmentsTargetBesideSource(target, source);
 }
 
 /**
@@ -302,6 +343,22 @@ private void addSegmentsFor(Vertex vertex, Obstacle obs) {
 			seg = new Segment(vertex, obs.topLeft);
 			seg2 = new Segment(vertex, obs.bottomLeft);
 			break;
+		default:
+			if (vertex.x == obs.x) {
+				seg = new Segment(vertex, obs.topLeft);
+				seg2 = new Segment(vertex, obs.bottomLeft);
+			} else if (vertex.y == obs.y) {
+				seg = new Segment(vertex, obs.topLeft);
+				seg2 = new Segment(vertex, obs.topRight);
+			} else if (vertex.y == obs.bottom() - 1) {
+				seg = new Segment(vertex, obs.bottomLeft);
+				seg2 = new Segment(vertex, obs.bottomRight);
+			} else if (vertex.x == obs.right() - 1) {
+				seg = new Segment(vertex, obs.topRight);
+				seg2 = new Segment(vertex, obs.bottomRight);
+			} else {
+				throw new RuntimeException("Unexpected vertex conditions"); //$NON-NLS-1$
+			}
 	}
 	
 	stack.push(obs);
@@ -312,6 +369,103 @@ private void addSegmentsFor(Vertex vertex, Obstacle obs) {
 	stack.push(seg2);
 }
 
+private void addSegmentsTargetAboveSource(Obstacle source, Obstacle target) {
+	//target located above source
+	Segment seg = null;
+	Segment seg2 = null;
+	if (target.x > source.x) {
+		seg = new Segment(source.topLeft, target.topLeft);
+		if (target.x < source.right() - 1)
+			seg2 = new Segment(source.topRight, target.bottomLeft);
+		else
+			seg2 = new Segment(source.bottomRight, target.topLeft);		
+	} else if (source.x == target.x) {
+		seg = new Segment(source.topLeft, target.bottomLeft);
+		seg2 = new Segment(source.topRight, target.bottomLeft);
+	} else {
+		seg = new Segment(source.bottomLeft, target.bottomLeft);
+		seg2 = new Segment(source.topRight, target.bottomLeft);
+	}
+	
+	stack.push(source);
+	stack.push(target);
+	stack.push(seg);
+	stack.push(source);
+	stack.push(target);
+	stack.push(seg2);
+	seg = null;
+	seg2 = null;
+	
+	if (target.right() < source.right()) {
+		seg = new Segment(source.topRight, target.topRight);
+		if (target.right() - 1 > source.x)
+			seg2 = new Segment(source.topLeft, target.bottomRight);
+		else
+			seg2 = new Segment(source.bottomLeft, target.topRight);
+	} else if (source.right() == target.right()) {
+		seg = new Segment(source.topRight, target.bottomRight);
+		seg2 = new Segment(source.topLeft, target.bottomRight);
+	} else {
+		seg = new Segment(source.bottomRight, target.bottomRight);
+		seg2 = new Segment(source.topLeft, target.bottomRight);
+	}	
+
+	stack.push(source);
+	stack.push(target);
+	stack.push(seg);
+	stack.push(source);
+	stack.push(target);
+	stack.push(seg2);
+}
+
+private void addSegmentsTargetBesideSource(Obstacle source, Obstacle target) {
+	//target located above source
+	Segment seg = null;
+	Segment seg2 = null;
+	if (target.y > source.y) {
+		seg = new Segment(source.topLeft, target.topLeft);
+		if (target.y < source.bottom() - 1)
+			seg2 = new Segment(source.bottomLeft, target.topRight);
+		else
+			seg2 = new Segment(source.bottomRight, target.topLeft);
+	} else if (source.y == target.y) {
+		//degenerate case
+		seg = new Segment(source.topLeft, target.topRight);
+		seg2 = new Segment(source.bottomLeft, target.topRight);
+	} else {
+		seg = new Segment(source.topRight, target.topRight);
+		seg2 = new Segment(source.bottomLeft, target.topRight);	
+	}
+	stack.push(source);
+	stack.push(target);
+	stack.push(seg);
+	stack.push(source);
+	stack.push(target);
+	stack.push(seg2);
+	seg = null;
+	seg2 = null;
+	
+	if (target.bottom() < source.bottom()) {
+		seg = new Segment(source.bottomLeft, target.bottomLeft);
+		if (target.bottom() - 1 > source.y)
+			seg2 = new Segment(source.topLeft, target.bottomRight);
+		else
+			seg2 = new Segment(source.topRight, target.bottomLeft);
+	} else if (source.bottom() == target.bottom()) {
+		seg = new Segment(source.bottomLeft, target.bottomRight);
+		seg2 = new Segment(source.topLeft, target.bottomRight);
+	} else {
+		seg = new Segment(source.bottomRight, target.bottomRight);
+		seg2 = new Segment(source.topLeft, target.bottomRight);
+	}
+	stack.push(source);
+	stack.push(target);
+	stack.push(seg);
+	stack.push(source);
+	stack.push(target);
+	stack.push(seg2);
+}
+
 /**
  * 
  */
@@ -343,7 +497,7 @@ private boolean determineShortestPath() {
 	if (!labelGraph())
 		return false;
 	Vertex vertex = end;
-	prevCost = end.cost;
+	prevCostRatio = end.cost / start.getDistance(end);
 	
 	Vertex nextVertex;
 	while (!vertex.equals(start)) {
@@ -365,13 +519,13 @@ private boolean determineShortestPath() {
 void fullReset() {
 	visibleVertices.clear();
 	segments.clear();
-	if (prevCost == 0) {
-		distance = start.getDistance(end);
+	if (prevCostRatio == 0) {
+		double distance = start.getDistance(end);
 		threshold = distance * OVAL_CONSTANT;
 	} else
-		threshold = prevCost * EPSILON;
+		threshold = prevCostRatio * EPSILON * start.getDistance(end);
 	visibleObstacles.clear();
-	reset();
+	resetPartial();
 }
 
 /**
@@ -391,9 +545,10 @@ boolean generateShortestPath(List allObstacles) {
 }
 
 /**
- * Returns the list of constrained points through which this path must pass.  See also
- * {@link #setBendPoints(PointList)}.
- * @return list of bend points.
+ * Returns the list of constrained points through which this path must pass or
+ * <code>null</code>.  
+ * @see #setBendPoints(PointList)
+ * @return list of bend points
  */
 public PointList getBendPoints() {
 	return bendpoints;
@@ -433,10 +588,13 @@ public Point getStartPoint() {
 Path getSubPath(Segment currentSegment) {
 	// ready new path
 	Path newPath = new Path(currentSegment.start, end);
-	newPath.grownSegments = new ArrayList(grownSegments.subList(grownSegments.indexOf(currentSegment), grownSegments.size()));
+	newPath.grownSegments = new ArrayList(grownSegments.subList(
+			grownSegments.indexOf(currentSegment),
+			grownSegments.size()));
 	
 	// fix old path
-	grownSegments = new ArrayList(grownSegments.subList(0, grownSegments.indexOf(currentSegment) + 1));		
+	grownSegments = new ArrayList(grownSegments.subList(
+			0, grownSegments.indexOf(currentSegment) + 1));		
 	end = currentSegment.end;
 	
 	subPath = newPath;
@@ -553,7 +711,26 @@ void refreshExcludedObstacles(List allObstacles) {
 	
 	for (int i = 0; i < allObstacles.size(); i++) {
 		Obstacle o = (Obstacle)allObstacles.get(i);
-		o.exclude = o.contains(start) || o.contains(end);
+		o.exclude = false;
+		
+		if (o.contains(start)) {
+			if (o.containsProper(start))
+				o.exclude = true;
+			else {
+				/*
+				 * $TODO Check for corners.  If the path begins exactly at the corner of
+				 * an obstacle, the exclude should also be true.
+				 */
+			}
+		}
+		
+		if (o.contains(end)) {
+			if (o.containsProper(end))
+				o.exclude = true;
+			else {
+				//check for corners.  See above statement.
+			}
+		}
 
 		if (o.exclude && !excludedObstacles.contains(o))
 			excludedObstacles.add(o);
@@ -563,7 +740,7 @@ void refreshExcludedObstacles(List allObstacles) {
 /**
  * Resets the fields for everything in the solve after the visibility graph steps.
  */
-void reset() {
+void resetPartial() {
 	isMarked = false;
 	isInverted = false;
 	subPath = null;
@@ -579,7 +756,7 @@ void reset() {
  * before it knew it was inverted.
  * @param currentSegment the segment at which the path found it was inverted
  */
-void resetVertices(Segment currentSegment) {
+void invertPriorVertices(Segment currentSegment) {
 	int stop = grownSegments.indexOf(currentSegment);
 	for (int i = 0; i < stop; i++) {
 		Vertex vertex = ((Segment)grownSegments.get(i)).end;
@@ -621,111 +798,16 @@ public void setStartPoint(Point start) {
 	isDirty = true;
 }
 
-private void targetAboveSource(Obstacle source, Obstacle target) {
-	//target located above source
-	Segment seg = null;
-	Segment seg2 = null;
-	if (target.x > source.x) {
-		seg = new Segment(source.topLeft, target.topLeft);
-		if (target.x < source.right() - 1)
-			seg2 = new Segment(source.topRight, target.bottomLeft);
-		else
-			seg2 = new Segment(source.bottomRight, target.topLeft);		
-	} else if (source.x == target.x) {
-		seg = new Segment(source.topLeft, target.bottomLeft);
-		seg2 = new Segment(source.topRight, target.bottomLeft);
-	} else {
-		seg = new Segment(source.bottomLeft, target.bottomLeft);
-		seg2 = new Segment(source.topRight, target.bottomLeft);
-	}
-	
-	stack.push(source);
-	stack.push(target);
-	stack.push(seg);
-	stack.push(source);
-	stack.push(target);
-	stack.push(seg2);
-	seg = null;
-	seg2 = null;
-	
-	if (target.right() < source.right()) {
-		seg = new Segment(source.topRight, target.topRight);
-		if (target.right() - 1 > source.x)
-			seg2 = new Segment(source.topLeft, target.bottomRight);
-		else
-			seg2 = new Segment(source.bottomLeft, target.topRight);
-	} else if (source.right() == target.right()) {
-		seg = new Segment(source.topRight, target.bottomRight);
-		seg2 = new Segment(source.topLeft, target.bottomRight);
-	} else {
-		seg = new Segment(source.bottomRight, target.bottomRight);
-		seg2 = new Segment(source.topLeft, target.bottomRight);
-	}	
-
-	stack.push(source);
-	stack.push(target);
-	stack.push(seg);
-	stack.push(source);
-	stack.push(target);
-	stack.push(seg2);
-}
-
-private void targetBesideSource(Obstacle source, Obstacle target) {
-	//target located above source
-	Segment seg = null;
-	Segment seg2 = null;
-	if (target.y > source.y) {
-		seg = new Segment(source.topLeft, target.topLeft);
-		if (target.y < source.bottom() - 1)
-			seg2 = new Segment(source.bottomLeft, target.topRight);
-		else
-			seg2 = new Segment(source.bottomRight, target.topLeft);
-	} else if (source.y == target.y) {
-		//degenerate case
-		seg = new Segment(source.topLeft, target.topRight);
-		seg2 = new Segment(source.bottomLeft, target.topRight);
-	} else {
-		seg = new Segment(source.topRight, target.topRight);
-		seg2 = new Segment(source.bottomLeft, target.topRight);	
-	}
-	stack.push(source);
-	stack.push(target);
-	stack.push(seg);
-	stack.push(source);
-	stack.push(target);
-	stack.push(seg2);
-	seg = null;
-	seg2 = null;
-	
-	if (target.bottom() < source.bottom()) {
-		seg = new Segment(source.bottomLeft, target.bottomLeft);
-		if (target.bottom() - 1 > source.y)
-			seg2 = new Segment(source.topLeft, target.bottomRight);
-		else
-			seg2 = new Segment(source.topRight, target.bottomLeft);
-	} else if (source.bottom() == target.bottom()) {
-		seg = new Segment(source.bottomLeft, target.bottomRight);
-		seg2 = new Segment(source.topLeft, target.bottomRight);
-	} else {
-		seg = new Segment(source.bottomRight, target.bottomRight);
-		seg2 = new Segment(source.topLeft, target.bottomRight);
-	}
-	stack.push(source);
-	stack.push(target);
-	stack.push(seg);
-	stack.push(source);
-	stack.push(target);
-	stack.push(seg2);
-}
-
 /**
- * Returns <code>true</code> if the path intersects the given obstacle.  Also dirties
- * the path.
+ * Returns <code>true</code> if the path is clean and intersects the given obstacle. Also
+ * dirties the path in the process.
  * @since 3.0
  * @param obs the obstacle
- * @return <code>true</code> if the path touches the obstacle
+ * @return <code>true</code> if a clean path touches the obstacle
  */
- boolean testObstacle(Obstacle obs) {
+ boolean testAndSet(Obstacle obs) {
+ 	if (isDirty)
+ 		return false;
  	//This will never actually happen because obstacles are not stored by identity
 	if (excludedObstacles.contains(obs))
 		return false;
@@ -746,4 +828,4 @@ private void targetBesideSource(Obstacle source, Obstacle target) {
 	return false;
 }
 
-}
\ No newline at end of file
+}
