@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -29,7 +29,6 @@ import org.eclipse.gef.EditPartFactory;
 import org.eclipse.gef.EditPartListener;
 import org.eclipse.gef.EditPartViewer;
 import org.eclipse.gef.EditPolicy;
-import org.eclipse.gef.GEF;
 import org.eclipse.gef.Request;
 import org.eclipse.gef.RequestConstants;
 import org.eclipse.gef.RootEditPart;
@@ -287,33 +286,18 @@ protected void deactivateEditPolicies() {
  * This method will log a message to GEF's trace/debug system if the corresponding flag
  * for EditParts is set to true.
  * @param message a debug message
+ * @deprecated in 3.1
  */
 protected final void debug(String message) {
-	if (!GEF.DebugEditParts)
-		return;
-	GEF.debug("EDITPART:\t" + toString() + ":\t" + message);//$NON-NLS-2$//$NON-NLS-1$
-}
-
-void debugPop() {
-	if (!GEF.DebugEditParts)
-		return;
-	GEF.debugPop();
-}
-
-void debugPush(String heading) {
-	if (GEF.DebugEditParts)
-		GEF.debugPush("EDITPART:\t" + toString()//$NON-NLS-1$
-			+ ": " + heading);//$NON-NLS-1$
 }
 
 /**
  * This method will log the message to GEF's trace/debug system if the corrseponding flag
  * for FEEDBACK is set to true.
  * @param message  Message to be passed
+ * @deprecated in 3.1
  */
 protected final void debugFeedback(String message) {
-	if (GEF.DebugFeedback)
-		GEF.debug("FEEDBACK:\t" + toString() + ":\t" + message);//$NON-NLS-2$//$NON-NLS-1$
 }
 
 /**
@@ -703,14 +687,14 @@ public void refresh() {
 /**
  * Updates the set of children EditParts so that it is in sync with the model children.
  * This method is called from {@link #refresh()}, and may also be called in response to
- * notification from the model.  This method requires linear time complete.  Clients
+ * notification from the model.  This method requires linear time to complete.  Clients
  * should call this method as few times as possible.  Consider also calling {@link
  * #removeChild(EditPart)} and {@link #addChild(EditPart, int)} which run in constant
  * time.
  * <P>
  * The update is performed by comparing the exising EditParts with the set of model
- * children returned from {@link #getModelChildren()}. EditParts whose model no longer
- * exists are {@link #removeChild(EditPart) removed}. New models have their EditParts
+ * children returned from {@link #getModelChildren()}. EditParts whose models no longer
+ * exist are {@link #removeChild(EditPart) removed}. New models have their EditParts
  * {@link #createChild(Object) created}.
  * <P>
  * This method should <em>not</em> be overridden.
