@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2004 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2004, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -13,6 +13,9 @@ package org.eclipse.gef.ui.palette;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.beans.PropertyChangeSupport;
+import java.util.ArrayList;
+import java.util.Iterator;
+import java.util.List;
 
 import org.eclipse.swt.SWT;
 import org.eclipse.swt.accessibility.ACC;
@@ -29,8 +32,8 @@ import org.eclipse.swt.events.MouseEvent;
 import org.eclipse.swt.events.MouseMoveListener;
 import org.eclipse.swt.events.MouseTrackAdapter;
 import org.eclipse.swt.events.MouseTrackListener;
-import org.eclipse.swt.graphics.Cursor;
 import org.eclipse.swt.graphics.Font;
+import org.eclipse.swt.graphics.FontData;
 import org.eclipse.swt.graphics.GC;
 import org.eclipse.swt.graphics.Image;
 import org.eclipse.swt.graphics.Point;
@@ -58,12 +61,12 @@ import org.eclipse.jface.util.TransferDropTargetListener;
 import org.eclipse.ui.IMemento;
 import org.eclipse.ui.IPerspectiveDescriptor;
 import org.eclipse.ui.IPerspectiveListener;
-import org.eclipse.ui.IViewPart;
+import org.eclipse.ui.IViewReference;
 import org.eclipse.ui.IWorkbenchPage;
 import org.eclipse.ui.IWorkbenchWindow;
+import org.eclipse.ui.PlatformUI;
 import org.eclipse.ui.XMLMemento;
 import org.eclipse.ui.internal.DragCursors;
-import org.eclipse.ui.internal.Workbench;
 
 import org.eclipse.draw2d.ActionEvent;
 import org.eclipse.draw2d.ActionListener;
@@ -104,6 +107,8 @@ public class FlyoutPaletteComposite
 	extends Composite
 {
 	
+private static final FontManager FONT_MGR = new FontManager();
+	
 private static final String PROPERTY_PALETTE_WIDTH
 		= "org.eclipse.gef.ui.palette.fpa.paletteWidth"; //$NON-NLS-1$
 private static final String PROPERTY_STATE
@@ -117,12 +122,17 @@ private static final int MAX_PALETTE_SIZE = 500;
 
 private static final int STATE_HIDDEN = 8;
 private static final int STATE_EXPANDED = 1;
-/*
- * This is the default state, i.e., when the palette view is not visible, if no initial
- * state is provided, the flyout palette will show up as collapsed.
+/**
+ * One of the two possible initial states of the flyout palette.  This is the default one.
+ * When in this state, only the flyout palette's sash is visible.
  */
-private static final int STATE_COLLAPSED = 2;
-private static final int STATE_PINNED_OPEN = 4;
+public static final int STATE_COLLAPSED = 2;
+/**
+ * One of the two possible initial states of the flyout palette.  When in this state,
+ * the flyout palette is completely visible and pinned open so that it doesn't disappear
+ * when the user wanders away from the flyout.
+ */
+public static final int STATE_PINNED_OPEN = 4;
 
 private static final Image LEFT_ARROW = new Image(null, ImageDescriptor.createFromFile(
 		Internal.class, "icons/palette_left.gif").getImageData()); //$NON-NLS-1$
@@ -136,10 +146,21 @@ private IMemento capturedPaletteState;
 private Control graphicalControl, sash;
 private PaletteViewerProvider provider;
 private FlyoutPreferences prefs;
+private Point cachedBounds = new Point(0, 0);
+/*
+ * Fix for Bug# 71525
+ * transferFocus is used to transfer focus from the button in the vertical sash title
+ * to the button in the horizontal paletteComposite title.  When either button is pressed
+ * it is set to true, and when either the sash or the paletteComposite gets notified of 
+ * the change in state, they transfer the focus to their button if this flag is set to
+ * true and if that button is visible.
+ */
+private boolean transferFocus = false;
 private int dock = PositionConstants.EAST;
-private int paletteState = -1;
+private int paletteState = STATE_HIDDEN;
 private int paletteWidth = DEFAULT_PALETTE_SIZE;
 private int minWidth = MIN_PALETTE_SIZE;
+private int cachedSize = -1, cachedState = -1, cachedLocation = -1;
 
 private IPerspectiveListener perspectiveListener = new IPerspectiveListener() {
 	public void perspectiveActivated(IWorkbenchPage page, 
@@ -156,7 +177,7 @@ private IPerspectiveListener perspectiveListener = new IPerspectiveListener() {
  * Constructor
  * 
  * @param	parent		The parent Composite
- * @param	style		The style of the widget to construct
+ * @param	style		The style of the widget to construct; only SWT.BORDER is allowed
  * @param	page		The current workbench page
  * @param	pvProvider	The provider that is to be used to create the flyout palette
  * @param	preferences	To save/retrieve the preferences for the flyout
@@ -171,13 +192,11 @@ public FlyoutPaletteComposite(Composite parent, int style, IWorkbenchPage page,
 	hookIntoWorkbench(page.getWorkbenchWindow());
 
 	// Initialize the state properly
+	if (prefs.getPaletteWidth() <= 0)
+		prefs.setPaletteWidth(DEFAULT_PALETTE_SIZE);
 	setPaletteWidth(prefs.getPaletteWidth());
 	setDockLocation(prefs.getDockLocation());
-	IViewPart part = page.findView(PaletteView.ID);
-	if (part == null)
-		setState(prefs.getPaletteState());
-	else
-		setState(STATE_HIDDEN);
+	updateState(page);
 
 	addListener(SWT.Resize, new Listener() {
 		public void handleEvent(Event event) {
@@ -188,9 +207,8 @@ public FlyoutPaletteComposite(Composite parent, int style, IWorkbenchPage page,
 			 * when the editor is closed or maximized.  We have to ignore such resizes.
 			 * See Bug# 62748
 			 */
-			if (area.width < minWidth)
-				return;
-			layout(true);
+			if (area.width > minWidth)
+				layout(true);
 		}
 	});
 
@@ -221,7 +239,17 @@ private void addListenerToCtrlHierarchy(Control parent, int eventType,
 
 private IMemento capturePaletteState(PaletteViewer viewer) {
 	IMemento memento = XMLMemento.createWriteRoot("paletteState"); //$NON-NLS-1$
-	viewer.saveState(memento);
+	try {
+		viewer.saveState(memento);
+	} catch (RuntimeException re) {
+		// Bug 74843 -- See comment #1
+		// If there's a problem with saving the palette's state, it simply won't be
+		// transferred to the new palette
+		memento = null;
+		/*
+		 * @TODO:Pratik   You should log this exception.
+		 */
+	}
 	return memento;
 }
 
@@ -234,7 +262,7 @@ private Composite createPaletteContainer() {
 }
 
 private Control createSash() {
-	return new Sash(this);
+	return new Sash(this, SWT.NONE);
 }
 
 private Control createTitle(Composite parent, boolean isHorizontal) {
@@ -252,49 +280,46 @@ private void handleEditorMaximized() {
 }
 
 private void handleEditorMinimized() {
-	handlePerspectiveActivated(
-			Workbench.getInstance().getActiveWorkbenchWindow().getActivePage(), null);
+	updateState(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage());
 }
 
 private void handlePerspectiveActivated(IWorkbenchPage page, 
 		IPerspectiveDescriptor perspective) {
-	IViewPart view = page.findView(PaletteView.ID);
-	if (view == null && isInState(STATE_HIDDEN))
-		setState(prefs.getPaletteState());
-	if (view != null && !isInState(STATE_HIDDEN))
-		setState(STATE_HIDDEN);
+	updateState(page);
 }
 
 private void handlePerspectiveChanged(IWorkbenchPage page, 
 		IPerspectiveDescriptor perspective, String changeId) {
 	if (changeId.equals(IWorkbenchPage.CHANGE_VIEW_SHOW) 
 			|| changeId.equals(IWorkbenchPage.CHANGE_VIEW_HIDE))
-		handlePerspectiveActivated(page, perspective);
+		updateState(page);
 }
 
-
-/*
- * @TODO:Pratik   Need to redo this method.  It should return false if ancestor is
- * null.
- */
-// Will return false if descendant is null
+// Will return false if the ancestor or descendant is null
 private boolean isDescendantOf(Control ancestor, Control descendant) {
-	if (descendant == null)
+	if (ancestor == null || descendant == null)
 		return false;
-	if (ancestor == descendant)
-		return true;
-	return isDescendantOf(ancestor, descendant.getParent());
+	while (descendant != null) {
+		if (ancestor == descendant)
+			return true;
+		descendant = descendant.getParent();
+	}
+	return false;
 }
 
 private boolean isInState(int state) {
 	return (paletteState & state) != 0;
 }
 
+private boolean isMirrored() {
+	return (getStyle() & SWT.MIRRORED) != 0;
+}
+
 /**
  * @see	Composite#layout(boolean)
  */
 public void layout(boolean changed) {
-	if (graphicalControl == null || graphicalControl.isDisposed())
+	if (graphicalControl == null || graphicalControl.isDisposed()) 
 		return;
 	
 	Rectangle area = getClientArea();
@@ -307,6 +332,19 @@ public void layout(boolean changed) {
 	pWidth = Math.max(pWidth, minWidth);
 	pWidth = Math.min(pWidth, maxWidth);
 	
+	/*
+	 * Fix for Bug# 65892
+	 * Laying out only when necessary helps reduce flicker on GTK in the case where the 
+	 * flyout palette is being resized past its maximum size.
+	 */
+	if (paletteState == cachedState && pWidth == cachedSize && cachedLocation == dock 
+			&& cachedBounds == getSize())
+		return;
+	cachedState = paletteState;
+	cachedSize = pWidth;
+	cachedLocation = dock;
+	cachedBounds = getSize();
+	
 	setRedraw(false);
 	if (isInState(STATE_HIDDEN)) {
 		sash.setVisible(false);
@@ -384,6 +422,19 @@ private void hookIntoWorkbench(final IWorkbenchWindow window) {
 	});
 }
 
+private boolean restorePaletteState(PaletteViewer newPalette, IMemento state) {
+	if (state != null) {
+		try {
+			return newPalette.restoreState(state);
+		} catch (RuntimeException re) {
+			/*
+			 * @TODO:Pratik  You should log this exception
+			 */
+		}
+	}
+	return false;
+}
+
 /**
  * If an external palette viewer is provided, palette state (that is captured in {@link
  * PaletteViewer#saveState(IMemento)} -- active tool, drawer expansion state, drawer pin
@@ -450,14 +501,18 @@ public void setGraphicalControl(Control graphicalViewer) {
 	});
 }
 
+/*
+ * @TODO:Pratik  For 4.0, change the parameter of this method to be EditpartViewer
+ * instead of GraphicalViewer.
+ */
 /**
  * This method hooks a DropTargetListener that collapses the flyout patette when the user
- * drags something from  the palette and moves the cursor to the graphical viewer's
- * control.  If the auto-hide feature of the palette is to work properly, this method
- * should be called before any other drop target listeners are added to the graphical
- * viewer.
+ * drags something from  the palette and moves the cursor to the primary viewer's
+ * control.  If the auto-hide feature of the palette is to work properly when dragging, 
+ * this method should be called before any other drop target listeners are added to the 
+ * primary viewer.
  * 
- * @param	viewer	the primary graphical viewer
+ * @param	viewer	the primary viewer
  */
 public void hookDropTargetListener(GraphicalViewer viewer) {
 	viewer.addDropTargetListener(new TransferDropTargetListener() {
@@ -491,12 +546,12 @@ public void hookDropTargetListener(GraphicalViewer viewer) {
  */
 private void setState(int newState) {
 	/*
-	 * Fix for Bug# 69617
+	 * Fix for Bug# 69617 and Bug# 81248
 	 * FlyoutPreferences.getPaletteState() could return an invalid state if none is
 	 * stored.  In that case, we use the default state: STATE_COLLAPSED.
 	 */
-	if ((newState & 
-			(STATE_COLLAPSED | STATE_EXPANDED | STATE_HIDDEN | STATE_PINNED_OPEN)) == 0)
+	if (newState != STATE_HIDDEN && newState != STATE_PINNED_OPEN 
+			&& newState != STATE_EXPANDED)
 		newState = STATE_COLLAPSED;
 	if (paletteState == newState)
 		return;
@@ -510,12 +565,9 @@ private void setState(int newState) {
 				pViewer = provider.createPaletteViewer(paletteContainer);
 				if (externalViewer != null)
 					transferState(externalViewer, pViewer);
-				else if (capturedPaletteState != null) {
-					pViewer.restoreState(capturedPaletteState);
-					// @TODO:Pratik  this should be moved out of this else - if statement,
-					// just in case
-					capturedPaletteState = null;
-				}
+				else
+					restorePaletteState(pViewer, capturedPaletteState);
+				capturedPaletteState = null;
 				minWidth = Math.max(pViewer.getControl().computeSize(0, 0).x, 
 						MIN_PALETTE_SIZE);
 			}
@@ -549,7 +601,15 @@ private void setState(int newState) {
 }
 
 private void transferState(PaletteViewer src, PaletteViewer dest) {
-	dest.restoreState(capturePaletteState(src));
+	restorePaletteState(dest, capturePaletteState(src));
+}
+
+private void updateState(IWorkbenchPage page) {
+	IViewReference view = page.findViewReference(PaletteView.ID);
+	if (view == null && isInState(STATE_HIDDEN))
+		setState(prefs.getPaletteState());
+	if (view != null && !isInState(STATE_HIDDEN))
+		setState(STATE_HIDDEN);
 }
 
 /**
@@ -560,21 +620,25 @@ private void transferState(PaletteViewer src, PaletteViewer dest) {
  */
 public interface FlyoutPreferences {
 	/**
-	 * When there is no saved dock location, this method can return any int (preferrably
-	 * a non-positive int).
+	 * Should return {@link PositionConstants#EAST} or {@link PositionConstants#WEST}. 
+	 * Any other int will be ignored and the default dock location (EAST) will be 
+	 * used instead.
 	 * @return the saved dock location of the Palette
 	 */
 	int getDockLocation();
 	/**
-	 * When there is no saved state, this method can return any non-positive int.  
-	 * Undesired behaviour is possible if a positve int that coincides with one of the
-	 * state constants is returned.
-	 * @return	the saved state of the palette (collapsed or pinned open)
+	 * When there is no saved state, this method can return any non-positive int (which 
+	 * will result in the palette using the default state -- collapsed), or
+	 * {@link FlyoutPaletteComposite#STATE_COLLAPSED}, or 
+	 * {@link FlyoutPaletteComposite#STATE_PINNED_OPEN}
+	 * @return	the saved state of the palette
 	 */
 	int getPaletteState();
 	/**
 	 * When there is no saved width, this method can return any int (preferrably a
-	 * non-positive int).
+	 * non-positive int).  Returning a non-positive int will cause the palette to be
+	 * sized to the default size, whereas returning a postive int will find the
+	 * closest match in the valid range (>= minimum and <= maximum)
 	 * @return	the saved width of the flyout palette
 	 */
 	int getPaletteWidth();
@@ -582,13 +646,15 @@ public interface FlyoutPreferences {
 	 * This method is invoked when the flyout palette's dock location is changed.  The
 	 * provided dock location should be persisted and returned in 
 	 * {@link #getDockLocation()}.
-	 * @param	location	an int representing the dock location
+	 * @param	location	{@link PositionConstants#EAST} or {@link PositionConstants#WEST}
 	 */
 	void setDockLocation(int location);
 	/**
-	 * This method is invoked when the flyout palette's default state is changed.  The
-	 * provided state should be persisted and returned in {@link #getPaletteState()}.
-	 * @param	state		an int the state of the flyout palette
+	 * This method is invoked when the flyout palette's state is changed (the new state
+	 * becomes the default).  The provided state should be persisted and returned in 
+	 * {@link #getPaletteState()}.
+	 * @param	state		{@link FlyoutPaletteComposite#STATE_COLLAPSED} or 
+	 * {@link FlyoutPaletteComposite#STATE_PINNED_OPEN}
 	 */
 	void setPaletteState(int state);
 	/**
@@ -601,8 +667,8 @@ public interface FlyoutPreferences {
 
 private class Sash extends Composite {
 	private Control button, title;
-	public Sash(Composite parent) {
-		super(parent, SWT.NONE);
+	public Sash(Composite parent, int style) {
+		super(parent, style);
 		button = createFlyoutControlButton(this);
 		title = createTitle(this, false);
 		new SashDragManager();
@@ -653,36 +719,32 @@ private class Sash extends Composite {
 	 * @see org.eclipse.swt.widgets.Composite#layout(boolean)
 	 */
 	public void layout(boolean changed) {
+		if (button == null || title == null)
+			return;
+		
 		if (isInState(STATE_PINNED_OPEN)) {
 			title.setVisible(false);
 			button.setVisible(false);
 			return;
 		}
 		
-		boolean stealFocus = !button.getVisible();
 		title.setVisible(true);
-			button.setVisible(true);
+		button.setVisible(true);
 		Rectangle area = getClientArea();
 		// 1 pixel margin all around to draw the raised border
 		area.x += 1;
 		area.y += 1;
 		area.width -= 2;
 		area.height -= 2;
-		if (button.getVisible()) {
-			button.setBounds(area.x, area.y, area.width, area.width);
-			// 5-pixel spacing
-			area.y += area.width + 3;
+		button.setBounds(area.x, area.y, area.width, area.width);
+		// 5-pixel spacing
+		area.y += area.width + 3;
+		title.setBounds(area.x, area.y, area.width, 
+				title.computeSize(-1, -1).y);
+		if (transferFocus) {
+			transferFocus = false;
+			button.setFocus();
 		}
-		if (title.getVisible()) {
-			title.setBounds(area.x, area.y, area.width, 
-					title.computeSize(-1, -1).y);
-		}
-		if (stealFocus)
-			Display.getCurrent().asyncExec(new Runnable() {
-				public void run() {
-					button.setFocus();
-				}
-			});
 	}
 	private void paintSash(GC gc) {
 		Rectangle bounds = getBounds();
@@ -703,6 +765,7 @@ private class Sash extends Composite {
 			implements MouseMoveListener {
 		protected boolean dragging = false;
 		protected boolean correctState = false;
+		protected boolean mouseDown = false;
 		protected int origX;
 		protected Listener keyListener = new Listener() {
 			public void handleEvent(Event event) {
@@ -721,12 +784,14 @@ private class Sash extends Composite {
 		public void mouseDown(MouseEvent me) {
 			if (me.button != 1)
 				return;
-			dragging = true;
+			mouseDown = true;
 			correctState = isInState(STATE_EXPANDED | STATE_PINNED_OPEN);
 			origX = me.x;
 			Display.getCurrent().addFilter(SWT.KeyDown, keyListener);
 		}
 		public void mouseMove(MouseEvent me) {
+			if (mouseDown)
+				dragging = true;
 			if (dragging && correctState)
 				handleSashDragged(me.x - origX);
 		}
@@ -740,6 +805,7 @@ private class Sash extends Composite {
 			}
 			dragging = false;
 			correctState = false;
+			mouseDown = false;
 		}
 	}
 }
@@ -783,19 +849,14 @@ private class TitleDragManager
 	public void handleEvent(Event event) {
 		dragging = true;
 		switchDock = false;
-		if (dock == PositionConstants.EAST)
-			threshold = Integer.MAX_VALUE / 2;
-		else
-			threshold = -1;
+		threshold = dock == PositionConstants.EAST ? Integer.MAX_VALUE / 2 : -1;
 		final Composite flyout = FlyoutPaletteComposite.this;
-		final int switchThreshold = flyout.toDisplay(flyout.getSize().x / 2, 0).x;
+		final Rectangle flyoutBounds = flyout.getBounds();
+		final int switchThreshold = flyoutBounds.x + (flyoutBounds.width / 2);
 		Rectangle bounds = sash.getBounds();
 		if (paletteContainer.getVisible())
 			bounds = bounds.union(paletteContainer.getBounds());
-		Point converted = flyout.toDisplay(bounds.x, bounds.y);
-		bounds.x = converted.x;
-		bounds.y = converted.y;
-		final Rectangle origBounds = bounds;
+		final Rectangle origBounds = Display.getCurrent().map(flyout, null, bounds);
 		final Tracker tracker = new Tracker(Display.getDefault(), SWT.NULL);
 		tracker.setRectangles(new Rectangle[] {origBounds});
 		tracker.setStippled(true);
@@ -804,8 +865,8 @@ private class TitleDragManager
 				Display.getCurrent().syncExec(new Runnable() {
 					public void run() {
 						Control ctrl = Display.getCurrent().getCursorControl();
-						Point pt = new Point(evt.x, evt.y);
-						switchDock = isDescendantOf(graphicalControl, ctrl) 
+						Point pt = flyout.toControl(evt.x, evt.y);
+						switchDock = isDescendantOf(graphicalControl, ctrl)
 								&& ((dock == PositionConstants.WEST && pt.x > threshold - 10)
 								|| (dock == PositionConstants.EAST && pt.x < threshold + 10));
 						boolean invalid = false;
@@ -826,24 +887,28 @@ private class TitleDragManager
 								placeHolder = new Rectangle(0, 0,
 										origBounds.width, origBounds.height);
 							else
-								placeHolder = new Rectangle(flyout.getBounds().width 
+								placeHolder = new Rectangle(flyoutBounds.width 
 										- origBounds.width, 0, origBounds.width, 
 										origBounds.height);
-							Point converted = flyout.toDisplay(
-									placeHolder.x, placeHolder.y);
-							placeHolder.x = converted.x;
-							placeHolder.y = converted.y;
+							placeHolder =
+									Display.getCurrent().map(flyout, null, placeHolder);
 						}
 						// update the cursor
-						Cursor cursor;
+						int cursor;
 						if (invalid)
-							cursor = DragCursors.getCursor(DragCursors.INVALID);
+							cursor = DragCursors.INVALID;
 						else if ((!switchDock && dock == PositionConstants.EAST)
 								|| (switchDock && dock == PositionConstants.WEST))
-							cursor = DragCursors.getCursor(DragCursors.RIGHT);
+							cursor = DragCursors.RIGHT;
 						else
-							cursor = DragCursors.getCursor(DragCursors.LEFT);
-						tracker.setCursor(cursor);
+							cursor = DragCursors.LEFT;
+						if (isMirrored()) {
+							if (cursor == DragCursors.RIGHT)
+								cursor = DragCursors.LEFT;
+							else if (cursor == DragCursors.LEFT)
+								cursor = DragCursors.RIGHT;
+						}
+						tracker.setCursor(DragCursors.getCursor(cursor));
 						// update the rectangle only if it has changed
 						if (!tracker.getRectangles()[0].equals(placeHolder))
 							tracker.setRectangles(new Rectangle[] {placeHolder});
@@ -860,8 +925,8 @@ private class TitleDragManager
 		}
 		tracker.dispose();
 	}
-	public void mouseEnter(MouseEvent e) {}
-	public void mouseExit(MouseEvent e) {}
+	public void mouseEnter(MouseEvent e) { }
+	public void mouseExit(MouseEvent e) { }
 	public void mouseHover(MouseEvent e) {
 		/*
 		 * @TODO:Pratik   Mouse hover events are received if the hover occurs just before 
@@ -935,15 +1000,11 @@ private class PaletteComposite extends Composite {
 		pViewer.getControl().setBounds(area);
 	}
 	protected void updateState() {
-		if (isInState(STATE_PINNED_OPEN)) {
-			title.setVisible(true);
-			if (!button.getVisible()) {
-				button.setVisible(true);
-				button.setFocus();
-			}
-		} else {
-			title.setVisible(false);
-			button.setVisible(false);
+		title.setVisible(isInState(STATE_PINNED_OPEN));
+		button.setVisible(isInState(STATE_PINNED_OPEN));
+		if (transferFocus && button.getVisible()) {
+			transferFocus = false;
+			button.setFocus();
 		}
 		layout(true);
 	}
@@ -954,7 +1015,7 @@ private class RotatedTitleLabel
 	public RotatedTitleLabel() {
 		FlyoutPaletteComposite.this.addDisposeListener(new DisposeListener() {
 			public void widgetDisposed(DisposeEvent e) {
-				if (getImage() != null)
+				if (getImage() != null && !getImage().isDisposed())
 					getImage().dispose();
 			}
 		});
@@ -967,7 +1028,7 @@ private class RotatedTitleLabel
 			graphics.drawFocus(0, 0, bounds.width - 1, bounds.height - 1);
 	}
 	public void setFont(Font f) {
-		if (f != font) {
+		if (f != getLocalFont()) {
 			super.setFont(f);
 			updateImage();
 		}
@@ -1032,15 +1093,18 @@ private static class TitleLabel extends Label {
 	}
 	protected void paintFigure(Graphics graphics) {
 		super.paintFigure(graphics);
-		if (hasFocus())
-			graphics.drawFocus(0, 0, bounds.width - 1, bounds.height - 1);
 		org.eclipse.draw2d.geometry.Rectangle area = 
 				getBounds().getCropped(super.getInsets());
 		org.eclipse.draw2d.geometry.Rectangle textBounds = getTextBounds();
 		// We reduce the width by 1 because FigureUtilities grows it by 1 unnecessarily
 		textBounds.width--;
+
+		if (hasFocus())
+			graphics.drawFocus(bounds.getResized(-1, -1)
+					.intersect(textBounds.getExpanded(getInsets())));
+		
 		int lineWidth = Math.min((area.width - textBounds.width - H_GAP * 2) / 2, 
-				LINE_LENGTH); 
+				LINE_LENGTH);
 		if (lineWidth >= MIN_LINE_LENGTH) {
 			int centerY = area.height / 2;
 			graphics.setForegroundColor(ColorConstants.buttonLightest);
@@ -1099,9 +1163,18 @@ private class ButtonCanvas extends Canvas {
 		return new org.eclipse.swt.graphics.Point(size.width, size.height);
 	}
 	private Image getButtonImage() {
+		Image arrow = null;
 		if (isInState(STATE_EXPANDED | STATE_PINNED_OPEN))
-			return dock == PositionConstants.WEST ? LEFT_ARROW : RIGHT_ARROW;
-		return dock == PositionConstants.WEST ? RIGHT_ARROW : LEFT_ARROW;
+			arrow = dock == PositionConstants.WEST ? LEFT_ARROW : RIGHT_ARROW;
+		else 
+			arrow = dock == PositionConstants.WEST ? RIGHT_ARROW : LEFT_ARROW;
+		if (isMirrored()) {
+			if (arrow == LEFT_ARROW)
+				arrow = RIGHT_ARROW;
+			else
+				arrow = LEFT_ARROW;
+		}
+		return arrow;	
 	}
 	private String getButtonTooltipText() {
 		if (isInState(STATE_COLLAPSED))
@@ -1117,6 +1190,7 @@ private class ButtonCanvas extends Canvas {
 		b.setBorder(new ButtonBorder(ButtonBorder.SCHEMES.TOOLBAR));
 		b.addActionListener(new ActionListener() {
 			public void actionPerformed(ActionEvent event) {
+				transferFocus = true;
 				if (isInState(STATE_COLLAPSED))
 					setState(STATE_PINNED_OPEN);
 				else
@@ -1185,7 +1259,6 @@ private class TitleCanvas extends Canvas {
 		else
 			fig = new RotatedTitleLabel();
 		final IFigure contents = fig;
-		contents.setFont(JFaceResources.getBannerFont());
 		contents.setRequestFocusEnabled(true);
 		contents.setFocusTraversable(true);
 		contents.addFocusListener(new FocusListener() {
@@ -1196,30 +1269,12 @@ private class TitleCanvas extends Canvas {
 				fe.loser.repaint();
 			}
 		});
-		final IPropertyChangeListener fontListener = new IPropertyChangeListener() {
-			public void propertyChange(org.eclipse.jface.util.PropertyChangeEvent event) {
-				if (JFaceResources.BANNER_FONT.equals(event.getProperty()))
-					contents.setFont(JFaceResources.getBannerFont());
-				if (isVisible()) {
-					redraw();
-					/*
-					 * If this canvas is in the sash, we want the FlyoutPaletteComposite
-					 * to layout (which will cause the sash to be resized and laid out).  
-					 * However, if this canvas is in the paletteContainer, the 
-					 * paletteContainer's bounds won't change, and hence it won't layout.
-					 * Thus, we also invoke getParent().layout().
-					 */
-					FlyoutPaletteComposite.this.layout(true);
-					getParent().layout(true);
-				}
-			}
-		};
-		JFaceResources.getFontRegistry().addListener(fontListener);
 		
 		lws = new LightweightSystem();
 		lws.setControl(this);
 		lws.setContents(contents);
 		setCursor(SharedCursors.SIZEALL);
+		FONT_MGR.register(this);
 		new TitleDragManager(this);
 		final MenuManager manager = new MenuManager();
 		MenuManager mgr = new MenuManager(PaletteMessages.DOCK_LABEL);
@@ -1239,7 +1294,7 @@ private class TitleCanvas extends Canvas {
 		
 		addDisposeListener(new DisposeListener() {
 			public void widgetDisposed(DisposeEvent e) {
-				JFaceResources.getFontRegistry().removeListener(fontListener);
+				FONT_MGR.unregister(TitleCanvas.this);
 				manager.dispose();
 			}
 		});		
@@ -1262,6 +1317,20 @@ private class TitleCanvas extends Canvas {
 			}
 		});
 	}
+	public void setFont(Font font) {
+		((IFigure)lws.getRootFigure().getChildren().get(0)).setFont(font);
+		if (isVisible()) {
+			/*
+			 * If this canvas is in the sash, we want the FlyoutPaletteComposite
+			 * to layout (which will cause the sash to be resized and laid out).  
+			 * However, if this canvas is in the paletteContainer, the 
+			 * paletteContainer's bounds won't change, and hence it won't layout.
+			 * Thus, we also invoke getParent().layout().
+			 */
+			FlyoutPaletteComposite.this.layout(true);
+			getParent().layout(true);
+		}
+	}
 }
 
 private class ChangeDockAction extends Action {
@@ -1293,4 +1362,71 @@ private class ChangeDockAction extends Action {
 	}
 }
 
-}
\ No newline at end of file
+private static class FontManager {
+	private final String fontName = getFontType();
+	private List registrants = new ArrayList();
+	private Font titleFont;
+	private boolean newFontCreated = false;
+	private final IPropertyChangeListener fontListener = new IPropertyChangeListener() {
+		public void propertyChange(org.eclipse.jface.util.PropertyChangeEvent event) {
+			if (fontName.equals(event.getProperty()))
+				handleFontChanged();
+		}
+	};
+	private FontManager() {
+	}
+	protected final Font createTitleFont() {
+		newFontCreated = false;
+		FontData[] data = JFaceResources.getFont(fontName).getFontData();
+		for (int i = 0; i < data.length; i++)
+			if ((data[i].getStyle() & SWT.BOLD) == 0) {
+				/*
+				 * @TODO:Pratik  need to test this in an environment where there are
+				 * multiple FontDatas for a font
+				 */
+				// Any problems with style settings (eg., in the case of a font that
+				// does not support bold case), will cause the font to ignore that style
+				// setting.
+				data[i].setStyle(data[i].getStyle() | SWT.BOLD);
+				newFontCreated = true;
+			}
+		if (newFontCreated)
+			return new Font(Display.getCurrent(), data);
+		return JFaceResources.getFont(fontName);
+	}
+	protected void dispose() {
+		if (newFontCreated && titleFont != null && !titleFont.isDisposed())
+			titleFont.dispose();
+		titleFont = null;
+		JFaceResources.getFontRegistry().removeListener(fontListener);
+	}
+	protected String getFontType() {
+		return JFaceResources.DIALOG_FONT;
+	}
+	protected void handleFontChanged() {
+		if (titleFont == null)
+			return;
+		Font oldFont = titleFont;
+		titleFont = createTitleFont();
+		for (Iterator iter = registrants.iterator(); iter.hasNext();)
+			((Control)iter.next()).setFont(titleFont);
+		oldFont.dispose();
+	}
+	protected void init() {
+		titleFont = createTitleFont();
+		JFaceResources.getFontRegistry().addListener(fontListener);		
+	}
+	public void register(Control ctrl) {
+		if (titleFont == null)
+			init();
+		ctrl.setFont(titleFont);
+		registrants.add(ctrl);
+	}
+	public void unregister(Control ctrl) {
+		registrants.remove(ctrl);
+		if (registrants.isEmpty())
+			dispose();
+	}
+}
+
+}
