@@ -1,10 +1,10 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2004 IBM Corporation and others.
- * All rights reserved. This program and the accompanying materials 
- * are made available under the terms of the Common Public License v1.0
+ * Copyright (c) 2000, 2005 IBM Corporation and others.
+ * All rights reserved. This program and the accompanying materials
+ * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
- * http://www.eclipse.org/legal/cpl-v10.html
- * 
+ * http://www.eclipse.org/legal/epl-v10.html
+ *
  * Contributors:
  *     IBM Corporation - initial API and implementation
  *******************************************************************************/
@@ -16,6 +16,8 @@ import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
+import java.util.HashSet;
+import java.util.Iterator;
 import java.util.List;
 import java.util.Map;
 
@@ -57,7 +59,7 @@ public abstract class AbstractEditPartViewer
 	implements EditPartViewer
 {
 
-private DisposeListener lDispose;
+private DisposeListener disposeListener;
 	
 /**
  * The raw list of selected editparts.
@@ -172,12 +174,12 @@ public void appendSelection(EditPart editpart) {
 }
 
 /**
- * @see EditPartViewer#createControl(org.eclipse.swt.widgets.Composite)
+ * @see EditPartViewer#createControl(Composite)
  */
 public abstract Control createControl(Composite parent);
 
 /**
- * @see EditPartViewer#deselect(org.eclipse.gef.EditPart)
+ * @see EditPartViewer#deselect(EditPart)
  */
 public void deselect(EditPart editpart) {
 	editpart.setSelected(EditPart.SELECTED_NONE);
@@ -397,7 +399,7 @@ public Map getVisualPartMap() {
  */
 protected void hookControl() {
 	Assert.isTrue(getControl() != null);
-	getControl().addDisposeListener(lDispose = new DisposeListener() {
+	getControl().addDisposeListener(disposeListener = new DisposeListener() {
 		public void widgetDisposed(DisposeEvent e) {
 			handleDispose(e);
 		}
@@ -714,22 +716,32 @@ public void setSelection(ISelection newSelection) {
 	if (!(newSelection instanceof IStructuredSelection))
 		return;
 	
-	List editparts = ((IStructuredSelection)newSelection).toList();
+	List orderedSelection = ((IStructuredSelection)newSelection).toList();
+	// Convert to HashSet to optimize performance.
+	Collection editparts = new HashSet(orderedSelection);
 	List selection = primGetSelectedEditParts();
 
 	setFocus(null);
-	for (int i = 0; i < selection.size(); i++)
-		((EditPart)selection.get(i)).setSelected(EditPart.SELECTED_NONE);
-	selection.clear();
-
-	for (int i = 0; i < editparts.size(); i++) {
-		EditPart part = (EditPart)editparts.get(i);
-		selection.add(part);
-		if (i == editparts.size() - 1)
-			part.setSelected(EditPart.SELECTED_PRIMARY);
-		else
-			part.setSelected(EditPart.SELECTED);
+	for (int i = 0; i < selection.size(); i++) {
+		EditPart part = (EditPart)selection.get(i);
+		if (!editparts.contains(part))
+			part.setSelected(EditPart.SELECTED_NONE);
 	}
+	selection.clear();
+	
+	if (!orderedSelection.isEmpty()) {
+		Iterator itr = orderedSelection.iterator();
+		while (true) {
+			EditPart part = (EditPart)itr.next();
+			selection.add(part);
+			if (!itr.hasNext()) {
+				part.setSelected(EditPart.SELECTED_PRIMARY);
+				break;
+			}
+			part.setSelected(EditPart.SELECTED);
+		}
+	}
+
 	fireSelectionChanged();
 }
 
@@ -739,9 +751,9 @@ public void setSelection(ISelection newSelection) {
 protected void unhookControl() {
 	Assert.isTrue(getControl() != null);
 	
-	if (lDispose != null) {
-		getControl().removeDisposeListener(lDispose);
-		lDispose = null;
+	if (disposeListener != null) {
+		getControl().removeDisposeListener(disposeListener);
+		disposeListener = null;
 	}
 	if (getContextMenu() != null)
 		getContextMenu().dispose();
@@ -756,4 +768,4 @@ protected void unhookControl() {
  */
 public void unregisterAccessibleEditPart(AccessibleEditPart acc) { }
 
-}
\ No newline at end of file
+}
